program HpglInterpreter;
(*$IKERNEL.INC*)
(*$IGRAPHLIB.INC*)
const
  TEST = false;
  START     = 'S';
  MNEMONIC  = 'M';
  PARAMETER = 'P';
  ERROR     = 'E';
  UNKNOWN   = 0;
  INIT      = 1;
  PEN_DOWN  = 2;
  PEN_UP    = 3;
  POS_ABS   = 4;
  POS_REL   = 5;
var
  FilVar:      Text;
  Ch:          Char;
  State:       Char;
  I:           Integer;
  FileName:    string(.14.);
  Command:     Integer;
  FirstMnemCh: Char;
  Value:       Integer;
  IsPenDown:   Boolean;
  LastX:       Integer;
  LastY:       Integer;
  ParamX:      Integer;
  ParamY:      Integer;
  ParamName:   Char;
  CmdCount:    Integer; (* TEST *)


(* forward declarations *)
procedure PrintCommand; forward;
procedure Draw;         forward;

(* functions and procedures *)
procedure InitParameter;
begin
  Value := 0;
end; (* InitParameter *)


function isWhitespace(c: Char): Boolean;
begin
  isWhitespace := ord(c) < 33;
end; (* isWhitespace *)


procedure FinishParameter;
begin
  if ParamName = 'X' then begin
    LastX := ParamX;
    ParamX := Value;
    ParamName := 'Y';
  end else begin
    LastY := ParamY;
    ParamY := Value;
    ParamName := 'X';
    if IsPenDown then Draw;
(*    if not IsPenDown then Draw; * TEST *)
  end;
  InitParameter;
end; (* FinishParameter *)


procedure ScaleParameter;
var r: Real;
begin
  r := ParamX;
  r := r / 10000 * 720;
  ParamX := trunc(r);

  r := 10000 - ParamY;
  r := r / 10000 * 256;
  ParamY := trunc(r);
end; (* ScaleParameter *)


procedure ProcessChar;
begin
  if Ch = ';' then begin
    State := START;
    FinishParameter;
    ParamName := 'X';
  end else begin
    case State of
      START: begin
          Command := UNKNOWN;
          InitParameter;
          if not isWhitespace(Ch) then begin
            if (Ch >= 'A') and (Ch <= 'Z') then begin
              FirstMnemCh := Ch;
              State := MNEMONIC;
            end else begin
              Writeln;
              Writeln('Invalid first character in Mnemonic: ', Ch);
              State := ERROR;
            end;
          end;
        end;
      MNEMONIC: begin
          if (Ch >= 'A') and (Ch <= 'Z') then begin
            case FirstMnemCh of
              'I': case Ch of
                     'N': begin Command := INIT;     IsPenDown := false; end;
                   end;
              'P': case Ch of
                     'D': begin Command := PEN_DOWN; IsPenDown := true;  end;
                     'U': begin Command := PEN_UP;   IsPenDown := false; end;
                     'A': begin Command := POS_ABS;                      end;
                     'R': begin Command := POS_REL;                      end;
                   end;
            end;
            State := PARAMETER;
          end else begin
            Writeln;
            Writeln('Invalid second character in Mnemonic: ', Ch);
            State := ERROR;
          end;
          CmdCount := CmdCount + 1;
        end;
      PARAMETER: begin
          if Ch <> '.' then begin (* real values not supported; dot ignored *)
            if (Ch = '-') then begin
              Value := -Value;
            end else if (Ch >= '0') and (Ch <='9') then begin
              Value := Value * 10 + Ord(Ch) - Ord('0');
            end else begin
              (* delimiter found: end parameter *)
              FinishParameter;
            end;
          end;
        end;
    end; (* case *)
  end;
end; (* ProcessChar *)


procedure PrintCommand;
begin
  case Command of
    UNKNOWN:  Write('UNKNOWN');
    PEN_DOWN: Write('PEN_DOWN');
    PEN_UP:   Write('PEN_UP');
    POS_ABS:  Write('POS_ABS');
    POS_REL:  Write('POS_REL');
  end;
end; (* PrintCommand *)


procedure Draw;
begin
  ScaleParameter;
  if (LastX >= 0) and (LastX < 720) and
     (LastY >= 0) and (LastY < 256) and
     (ParamX >= 0) and (ParamX < 720) and
     (ParamY >= 0) and (ParamY < 256) then begin
    line(LastX, LastY, ParamX, ParamY, 0);
  end;
(*
  PrintCommand;
  Writeln(' X:', ParamX, ' Y:', ParamY);
*)
end; (* Draw *)


(* Main program *)
begin
  ClrScr;
  if TEST then begin
    FileName := 'g2.plt';
  end
  else begin
    Write('Enter name of file to list: ');
    Readln(FileName);
  end;
  Assign(FilVar, FileName);
  (*$I-*)
  Reset(FilVar);
  (*$I+*)
  if IOResult <> 0 then
  begin
    Writeln('Datei nicht gefunden: ', FileName);
  end;

  State := START;
  ParamName := 'X';
  LastX := 0;
  LastY := 0;
  CmdCount := 0;

  ClrScr;
  GraphInit;
  HideCursor;

(*  while (not Eof(FilVar)) and (State <> ERROR) and (CmdCount < 30) do begin *)
  while (not Eof(FilVar)) and (State <> ERROR) do begin
    Read(FilVar, Ch);

(*
    if (Ord(Ch) = 10) or (Ord(Ch) = 13) then
       Writeln
    else
      Write(Ch);
*)
    ProcessChar;
  end; (* while *)

  repeat until KeyPressed;
  ShowCursor;
end.
